"""
Django settings for example project.

Generated by 'django-admin startproject' using Django 4.2.7.

For more information on this file, see
https://docs.djangoproject.com/en/4.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.2/ref/settings/
"""

from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-9zrly450y@f)_@pja!f@_jm30tot6)d7s@@@2sdso0&j9*9=ll'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    "django_multidatabase_queryset",
    "core",
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'example.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/4.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    },
    "db_cold": {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db_cold.sqlite3',
    },
}


# Password validation
# https://docs.djangoproject.com/en/4.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.2/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/4.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
LOG_DIR = BASE_DIR / 'log'
LOG_DIR.mkdir(exist_ok=True)
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': ('[%(levelname)5s] %(asctime)s %(pathname)s '
                       '%(funcName)s (line: %(lineno)d)'
                       '    %(message)s'),
        },
        'simple': {
            'format': '[%(levelname)s] %(message)s ',
        },
    },
    'handlers': {
        'error_file': {
            'level': "ERROR",
            'class': 'logging.FileHandler',
            'filename': LOG_DIR / 'error.log',
            'formatter': 'verbose',
        },
        'warning_file': {
            'level': "WARNING",
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': LOG_DIR / 'warning.log',
            'maxBytes': 1024 * 1024 * 10,
            'backupCount': 20,
            'formatter': 'verbose',
        },
        'info_file': {
            'level': "INFO",
            'class': 'logging.handlers.RotatingFileHandler',
            'maxBytes': 1024 * 1024 * 10,
            'backupCount': 20,
            'filename': LOG_DIR / 'info.log',
            'formatter': 'verbose',
        },
        'debug_file': {
            'level': "DEBUG",
            'class': 'logging.handlers.RotatingFileHandler',
            'maxBytes': 1024 * 1024 * 10,
            'backupCount': 20,
            'filename': LOG_DIR / 'debug.log',
            'formatter': 'verbose',
        },
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'default': {
            'handlers': ['debug_file', 'info_file',
                         'warning_file', 'error_file', 'console'],
            'level': "INFO",
        },
        'django': {
            'handlers': ['debug_file', 'info_file',
                         'warning_file', 'error_file', 'console'],
            'level': "INFO",
        },
        'testapp': {
            'handlers': ['debug_file', 'info_file',
                         'warning_file', 'error_file', 'console'],
            'level': "INFO",
        },
    },
}



# pylint: disable=missing-class-docstring, missing-function-docstring
import logging

from django.test import TestCase
from core.models import UserAction


logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class Test(TestCase):
    databases = ["default", "db_cold"]

    def test_all(self):
        hot_data = UserAction(type="hot")
        hot_data.save(using="default")
        cold_data = UserAction(type="cold")
        cold_data.save(using="db_cold")
        data = list(UserAction.objects.all())
        self.assertEqual(
                UserAction.objects.count(),
                2,
        )
        self.assertEqual(
                len(data), 
                2,
        )
        cold_data = list(UserAction.objects.filter(type="cold"))
        self.assertEqual(
                len(cold_data), 
                1,
        )

    def test_order(self):
        data1 = UserAction(id=1, type="type4")
        data1.save(using="default")
        data2 = UserAction(id=2, type="type3")
        data2.save(using="db_cold")
        data3 = UserAction(id=3, type="type1")
        data3.save(using="db_cold")
        data4 = UserAction(id=4, type="type2")
        data4.save(using="default")
        order_qs = UserAction.objects.order_by(
            "type", "pk")
        data = list(order_qs)
        LOGGER.info(data)
        self.assertEqual(data[0].type, "type1")
        self.assertEqual(data[1].type, "type2")
        self.assertEqual(data[2].type, "type3")
        self.assertEqual(data[3].type, "type4")
        self.assertEqual(order_qs.count(), 4)
        # breakpoint()
        exclude_qs = order_qs.exclude(id__gte=2)
        self.assertEqual(list(exclude_qs),
                         [data1])

    def test_order_again(self):
        """
        test if the queryset can update
        """
        order_qs2 = UserAction.objects.order_by(
            "type", "pk").filter(type="non exist")
        self.assertEquals(list(order_qs2), [])
        data1 = UserAction(id=1, type="non exist")
        data1.save(using="default")
        order_qs2 = order_qs2.filter()
        self.assertEquals(list(order_qs2), [data1])
        self.assertEquals(order_qs2.first(), data1)
        self.assertEquals(order_qs2.exists(), True)

    def test_using(self):
        hot_data = UserAction(type="hot")
        hot_data.save(using="default")
        cold_data = UserAction(type="cold")
        cold_data.save(using="db_cold")
        self.assertEqual(
            UserAction.objects.all().using("default").count(), 1)
        self.assertEqual(
            UserAction.objects.all().using("db_cold").count(), 1)
        self.assertEqual(
            UserAction.objects.using("db_cold").count(), 1)
        self.assertEqual(
            UserAction.objects.filter(type="hot").get(),
            UserAction.objects.get(type="hot"),
        )
        hot_data.delete()
        hot_data.save(using="db_cold")
        self.assertEqual(
            UserAction.objects.filter(type="hot").get(),
            UserAction.objects.get(type="hot"),
        )
